@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.h,v 1.7 2003/11/06 16:29:52 marco Exp $
+ *  $Id: ContentHandler.h,v 1.12 2004/01/21 23:39:42 marco Exp $
  */
 
 #ifndef __ContentHandler_h
@@ -40,6 +40,14 @@
 #include "nsISupports.h"
 #include "nsError.h"
 
+typedef enum
+{
+	CONTENT_ACTION_OPEN,
+	CONTENT_ACTION_DOWNLOAD,
+	CONTENT_ACTION_SAVEAS,
+	CONTENT_ACTION_NONE
+} ContentAction;
+
 #define G_CONTENTHANDLER_CID			     \
 { /* 16072c4a-23a6-4996-9beb-9335c06bbeae */         \
     0x16072c4a,                                      \
@@ -59,30 +67,26 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	GContentHandler();
 	virtual ~GContentHandler();
 
-	NS_METHOD FindHelperApp (void);
-	NS_METHOD LaunchHelperApp (void);
-
-	NS_METHOD GetLauncher (nsIHelperAppLauncher * *_retval);
-	NS_METHOD SetHelperApp(GnomeVFSMimeApplication *mHelperApp,
-			       PRBool alwaysUse);
-	NS_METHOD SynchroniseMIMEInfo (void);
-
   private:
-	/* additional members */
-	NS_METHOD Init (void);
-	NS_METHOD ProcessMimeInfo (void);
-	NS_METHOD MIMEAskAction (void);
-	
-	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
 
+	NS_METHOD Init ();
+	NS_METHOD LaunchHelperApp ();
+
+	NS_METHOD MIMEConfirmAction ();
+	NS_METHOD MIMEDoAction ();
+	NS_METHOD CheckAppSupportScheme ();
+
+	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
 	nsCOMPtr<nsIURI> mUri;
-	PRInt64 mTimeDownloadStarted;
 	nsCOMPtr<nsIFile> mTempFile;
+	nsCOMPtr<nsISupports> mContext;
 
 	char *mMimeType;	
-	PRBool mUrlHelper;
+	PRBool mAppSupportScheme;
 	GnomeVFSMimeApplication *mHelperApp;
-	
+	ContentAction mAction;
+	EphyMimePermission mPermission;
+
 	nsCString mUrl;
 	nsCString mScheme;
 };
