@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.16 2004/01/22 19:48:21 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.18 2004/01/27 12:48:55 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -190,7 +190,8 @@ EphyBrowser::EphyBrowser ()
 {
 	mFaviconEventListener = nsnull;
 	mPopupEventListener = nsnull;
-	mEventReceiver = nsnull;	
+	mEventReceiver = nsnull;
+	mInitialized = PR_FALSE;
 }
 
 EphyBrowser::~EphyBrowser ()
@@ -201,6 +202,8 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 {
 	nsresult rv;
 
+	if (mInitialized) return NS_OK;
+
 	gtk_moz_embed_get_nsIWebBrowser (mozembed,
 					 getter_AddRefs(mWebBrowser));
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
@@ -228,6 +231,8 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
  	rv = GetListener();
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
+	mInitialized = PR_TRUE;
+
 	return AttachListeners();
 }
 
@@ -900,7 +905,7 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 				/* Mozilla Bug 218277, 195946 and others */
 				default_text.ReplaceChar(0xa0, ' ');
 
-				if (Compare (user_text, default_text) != 0)
+				if (!user_text.Equals (default_text))
 				{
 					*aHasTextArea = PR_TRUE;
 					return NS_OK;
@@ -918,13 +923,25 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			if (type.EqualsIgnoreCase("text"))
 			{
 				nsAutoString default_text, user_text;
+				PRInt32 max_length;
 				inputElement->GetDefaultValue (default_text);
 				inputElement->GetValue (user_text);
+				inputElement->GetMaxLength (&max_length);
+
+				/* Guard against arguably broken forms where
+				 * default_text is longer than maxlength
+				 * (user_text is cropped, default_text is not)
+				 * Mozilla bug 232057
+				 */
+				if (default_text.Length() > (PRUint32)max_length)
+				{
+					default_text.Truncate (max_length);
+				}
 
 				/* Mozilla Bug 218277, 195946 and others */
 				default_text.ReplaceChar(0xa0, ' ');
 
-				if (Compare (user_text, default_text) != 0)
+				if (!user_text.Equals (default_text))
 				{
 					(*aNumTextFields)++;
 					if (*aNumTextFields >= NUM_MODIFIED_TEXTFIELDS_REQUIRED)
