@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.20 2004/01/12 19:31:40 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.23 2004/01/22 19:49:36 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -188,7 +188,7 @@ nsresult EventContext::ResolveDocumentURL (nsIDocument *doc, const nsAString &re
 }
 
 nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
-					EphyEmbedEvent *info)
+					MozillaEmbedEvent *info)
 {
 	nsresult rv;
 
@@ -635,7 +635,7 @@ nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAutoString& url)
 	return NS_OK;
 }
 
-nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, EphyEmbedEvent *info)
+nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, MozillaEmbedEvent *info)
 {
 	nsresult result;
 
@@ -718,7 +718,7 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, EphyEmb
 	return NS_OK;
 }
 
-nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, EphyEmbedEvent *info)
+nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, MozillaEmbedEvent *info)
 {
 	nsresult rv;
 
@@ -817,10 +817,12 @@ nsresult EventContext::CheckLinkScheme (const nsAString &link)
 	rv = uri->GetScheme (scheme);
 	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 
-	if (scheme.EqualsIgnoreCase ("http") ||
+	if (scheme.EqualsIgnoreCase ("http")  ||
 	    scheme.EqualsIgnoreCase ("https") ||
-	    scheme.EqualsIgnoreCase ("ftp") ||
-	    scheme.EqualsIgnoreCase ("file") ||
+	    scheme.EqualsIgnoreCase ("ftp")   ||
+	    scheme.EqualsIgnoreCase ("file")  ||
+	    scheme.EqualsIgnoreCase ("data")  ||
+	    scheme.EqualsIgnoreCase ("about") ||
 	    scheme.EqualsIgnoreCase ("gopher"))
 	{
 		SetIntProperty ("link-has-web-scheme", TRUE);
@@ -838,9 +840,7 @@ nsresult EventContext::SetIntProperty (const char *name, int value)
 	
 	g_value_set_int (val, value);
 
-	ephy_embed_event_set_property (mEmbedEvent, 
-			     	       name,
-			     	       val);
+	mozilla_embed_event_set_property (mEmbedEvent, name, val);
 
 	return NS_OK;
 }
@@ -853,9 +853,7 @@ nsresult EventContext::SetStringProperty (const char *name, const char *value)
 	
 	g_value_set_string (val, value);
 			 
-	ephy_embed_event_set_property (mEmbedEvent, 
-			     	       name,
-			     	       val);
+	mozilla_embed_event_set_property (mEmbedEvent, name, val);
 
 	return NS_OK;
 }
