@@ -38,7 +38,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.14 2004/01/10 11:19:52 chpe Exp $
+ * $Id: MozDownload.cpp,v 1.21 2004/01/29 11:43:27 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,12 +49,15 @@
 #include "mozilla-download.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-file-helpers.h"
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <glib/gi18n.h>
 
 #include "nsIExternalHelperAppService.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsIRequest.h"
+#include "nsIMIMEInfo.h"
 #include "netCore.h"
 
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
@@ -88,6 +91,7 @@ MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisp
 		   nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
 {
 	PRBool addToView = PR_TRUE;
+	nsresult rv;
 
 	if (mEmbedPersist)
 	{
@@ -106,6 +110,7 @@ MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisp
 	mPercentComplete = 0;
 	mInterval = 4000; /* in ms */
 	mLastUpdate = mStartTime;
+	mMIMEInfo = aMIMEInfo;
 
 	if (aPersist)
 	{
@@ -262,6 +267,8 @@ NS_IMETHODIMP
 MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 			    PRUint32 aStateFlags, nsresult aStatus)
 {
+	nsresult rv;
+
 	/* For a file download via the external helper app service, we will never get a start
            notification. The helper app service has gotten that notification before it created us. */
 	if (!mGotFirstStateChange)
@@ -302,6 +309,42 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 				mozilla_embed_persist_cancelled (mEmbedPersist);
 			}
 		}
+		else if (NS_SUCCEEDED (aStatus))
+		{
+			GnomeVFSMimeApplication *helperApp;
+			char *mimeType;	
+
+			rv = mMIMEInfo->GetMIMEType (&mimeType);
+			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+			helperApp = gnome_vfs_mime_get_default_application (mimeType); 
+
+			PRUnichar *description;
+			mMIMEInfo->GetApplicationDescription (&description);
+
+			/* HACK we use the application description to decide
+			   if we have to open the saved file */
+			if ((strcmp (NS_ConvertUCS2toUTF8 (description).get(), "gnome-default") == 0) &&
+			    helperApp)
+			{
+				GList *params = NULL;
+				char *param;
+				nsCAutoString aDest;
+
+				mDestination->GetNativePath (aDest);
+
+				param = gnome_vfs_make_uri_canonical (aDest.get ());
+				params = g_list_append (params, param);
+				gnome_vfs_mime_application_launch (helperApp, params);
+				g_free (param);
+
+				g_list_free (params);
+			}
+
+			nsMemory::Free (mimeType);
+			nsMemory::Free (description);
+			gnome_vfs_mime_application_free (helperApp);
+		}
 	}
         
 	return NS_OK; 
@@ -366,6 +409,12 @@ MozDownload::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *aReques
 void
 MozDownload::Cancel()
 {
+	if (mDownloadState != EPHY_DOWNLOAD_DOWNLOADING &&
+	    mDownloadState != EPHY_DOWNLOAD_PAUSED)
+	{
+		return;
+	}
+	
 	if (mWebPersist)
 	{
 		mWebPersist->CancelSave ();
@@ -491,40 +540,26 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 static char*
 GetFilePath (const char *filename)
 {
-	char *path, *download_dir;
+	char *path, *download_dir, *expanded, *tmp;
 
 	download_dir = eel_gconf_get_string (CONF_STATE_DOWNLOAD_DIR);
+
 	if (!download_dir)
 	{
 		/* Emergency download destination */
 		download_dir = g_strdup (g_get_home_dir ());
 	}
-
-	if (!strcmp (download_dir, "Desktop"))
+	else if (g_utf8_collate (download_dir, "Downloads") == 0)
 	{
-		if (eel_gconf_get_boolean (CONF_DESKTOP_IS_HOME_DIR))
-		{
-			path = g_build_filename 
-				(g_get_home_dir (),
-				 filename,
-				 NULL);
-		}
-		else
-		{
-			path = g_build_filename 
-				(g_get_home_dir (), "Desktop",
-				 filename,
-				 NULL);
-		}
+		tmp = ephy_file_downloads_dir ();
+		g_free (download_dir);
+		download_dir = tmp;
 	}
-	else
-	{
-		char *expanded;
 
-		expanded = gnome_vfs_expand_initial_tilde (download_dir);
-		path = g_build_filename (expanded, filename, NULL);
-		g_free (expanded);
-	}
+	expanded = gnome_vfs_expand_initial_tilde (download_dir);
+	path = g_build_filename (expanded, filename, NULL);
+	g_free (expanded);
+
 	g_free (download_dir);
 
 	return path;
