@@ -55,6 +55,8 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
+#define EGG_TOOLBAR_EDITOR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EGG_TYPE_TOOLBAR_EDITOR, EggToolbarEditorPrivate))
+
 struct EggToolbarEditorPrivate
 {
   GtkUIManager *merge;
@@ -94,6 +96,36 @@ egg_toolbar_editor_get_type (void)
   return egg_toolbar_editor_type;
 }
 
+static int
+compare_func (GtkAction *a, GtkAction *b)
+{
+  GValue value_a = { 0, }, value_b = { 0, };
+  const char *short_label_a, *short_label_b;
+  int ret;
+
+  g_value_init (&value_a, G_TYPE_STRING);
+  g_object_get_property (G_OBJECT (a), "short_label", &value_a);
+  short_label_a = g_value_get_string (&value_a);
+
+  g_value_init (&value_b, G_TYPE_STRING);
+  g_object_get_property (G_OBJECT (b), "short_label", &value_b);
+  short_label_b = g_value_get_string (&value_b);
+
+  ret = g_utf8_collate (short_label_a, short_label_b);
+
+  g_value_unset (&value_a);
+  g_value_unset (&value_b);
+
+  return ret;
+}
+
+static void
+sort_list (EggToolbarEditor *editor)
+{
+  editor->priv->actions_list = g_list_sort (editor->priv->actions_list,
+					    (GCompareFunc) compare_func);
+}
+
 static GtkAction *
 find_action (EggToolbarEditor *t,
 	     const char       *name)
@@ -103,7 +135,7 @@ find_action (EggToolbarEditor *t,
 
   l = gtk_ui_manager_get_action_groups (t->priv->merge);
 
-  g_return_val_if_fail (IS_EGG_TOOLBAR_EDITOR (t), NULL);
+  g_return_val_if_fail (EGG_IS_TOOLBAR_EDITOR (t), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
   for (; l != NULL; l = l->next)
@@ -123,18 +155,18 @@ egg_toolbar_editor_set_merge (EggToolbarEditor *t,
 			      GtkUIManager     *merge)
 {
   g_return_if_fail (GTK_IS_UI_MANAGER (merge));
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (t));
+  g_return_if_fail (EGG_IS_TOOLBAR_EDITOR (t));
 
-  t->priv->merge = merge;
+  t->priv->merge = g_object_ref (merge);
 }
 
 static void
 egg_toolbar_editor_set_model (EggToolbarEditor *t,
 			      EggToolbarsModel *model)
 {
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (t));
+  g_return_if_fail (EGG_IS_TOOLBAR_EDITOR (t));
 
-  t->priv->model = model;
+  t->priv->model = g_object_ref (model);
 }
 
 static void
@@ -198,19 +230,26 @@ egg_toolbar_editor_class_init (EggToolbarEditorClass *klass)
 				  g_param_spec_object ("ToolbarsModel",
 						       "ToolbarsModel",
 						       "Toolbars Model",
-						       EGG_TOOLBARS_MODEL_TYPE,
+						       EGG_TYPE_TOOLBARS_MODEL,
 						       G_PARAM_READWRITE));
+
+  g_type_class_add_private (object_class, sizeof (EggToolbarEditorPrivate));
 }
 
 static void
 egg_toolbar_editor_finalize (GObject *object)
 {
-  EggToolbarEditor *t = EGG_TOOLBAR_EDITOR (object);
+  EggToolbarEditor *editor = EGG_TOOLBAR_EDITOR (object);
 
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (object));
+  if (editor->priv->merge)
+    {
+      g_object_unref (editor->priv->merge);
+    }
 
-  g_free (t->priv);
+  if (editor->priv->model)
+    {
+      g_object_unref (editor->priv->model);
+    }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -219,16 +258,10 @@ GtkWidget *
 egg_toolbar_editor_new (GtkUIManager *merge,
 			EggToolbarsModel *model)
 {
-  EggToolbarEditor *t;
-
-  t = EGG_TOOLBAR_EDITOR (g_object_new (EGG_TOOLBAR_EDITOR_TYPE,
-					"MenuMerge", merge,
-					"ToolbarsModel", model,
-					NULL));
-
-  g_return_val_if_fail (t->priv != NULL, NULL);
-
-  return GTK_WIDGET (t);
+  return GTK_WIDGET (g_object_new (EGG_TYPE_TOOLBAR_EDITOR,
+				   "MenuMerge", merge,
+				   "ToolbarsModel", model,
+				   NULL));
 }
 
 static void
@@ -257,7 +290,7 @@ editor_drag_data_received_cb (GtkWidget          *widget,
 {
   GtkAction *action;
 
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (editor));
+  g_return_if_fail (EGG_IS_TOOLBAR_EDITOR (editor));
   g_return_if_fail (selection_data != NULL);
 
   action = find_action (editor, (const char *)selection_data->data);
@@ -268,6 +301,7 @@ editor_drag_data_received_cb (GtkWidget          *widget,
       editor->priv->actions_list = g_list_append
 	    (editor->priv->actions_list, action);
     }
+  sort_list (editor);
 
   update_editor_sheet (editor);
 }
@@ -278,7 +312,7 @@ editor_drag_data_delete_cb (GtkWidget          *widget,
 			    EggToolbarEditor *editor)
 {
   GtkAction *action;
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (editor));
+  g_return_if_fail (EGG_IS_TOOLBAR_EDITOR (editor));
 
   action = GTK_ACTION (g_object_get_data (G_OBJECT (widget), "egg-action"));
   if (action)
@@ -286,6 +320,7 @@ editor_drag_data_delete_cb (GtkWidget          *widget,
       editor->priv->actions_list = g_list_remove
 	    (editor->priv->actions_list, action);
     }
+  sort_list (editor);
 
   update_editor_sheet (editor);
 }
@@ -399,12 +434,16 @@ editor_create_item (EggToolbarEditor *editor,
   if (type == GTK_IMAGE_STOCK)
     {
       gchar *stock_id;
-      gtk_image_get_stock (icon, &stock_id, NULL); 
-      gtk_drag_source_set_icon_stock (event_box, stock_id);
+      GdkPixbuf *pixbuf;
+      gtk_image_get_stock (icon, &stock_id, NULL);
+      pixbuf = gtk_widget_render_icon (event_box, stock_id,
+	                               GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
+      gtk_drag_source_set_icon_pixbuf (event_box, pixbuf);
+      g_object_unref (pixbuf);
     }
   else if (type == GTK_IMAGE_PIXBUF)
     {
-      GdkPixbuf *pixbuf = gtk_image_get_pixbuf (icon);    
+      GdkPixbuf *pixbuf = gtk_image_get_pixbuf (icon);
       gtk_drag_source_set_icon_pixbuf (event_box, pixbuf);
     }
 
@@ -427,14 +466,14 @@ static void
 update_editor_sheet (EggToolbarEditor *editor)
 {
   GList *l;
-  GList *to_drag = editor->priv->actions_list;
+  GList *to_drag;
   int x, y, height, width;
   GtkWidget *table;
   GtkWidget *viewport;
   GtkWidget *item;
   GtkWidget *icon;
 
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (editor));
+  g_return_if_fail (EGG_IS_TOOLBAR_EDITOR (editor));
 
   viewport = GTK_BIN (editor->priv->scrolled_window)->child;
   if (viewport)
@@ -453,6 +492,8 @@ update_editor_sheet (EggToolbarEditor *editor)
   g_signal_connect (table, "drag_data_received",
 		    G_CALLBACK (editor_drag_data_received_cb), editor);
 
+  to_drag = editor->priv->actions_list;
+
   x = y = 0;
   width = 4;
   height = (g_list_length (to_drag) - 1) / width + 1;
@@ -463,7 +504,7 @@ update_editor_sheet (EggToolbarEditor *editor)
       GtkAction *action = (l->data);
       const char *stock_id, *short_label;
       GValue value = { 0, };
-                                                                                                                             
+
       g_value_init (&value, G_TYPE_STRING);
       g_object_get_property (G_OBJECT (action), "stock_id", &value);
       stock_id = g_value_get_string (&value);
@@ -505,7 +546,7 @@ setup_editor (EggToolbarEditor *editor)
   GtkWidget *image;
   GtkWidget *label;
 
-  g_return_if_fail (IS_EGG_TOOLBAR_EDITOR (editor));
+  g_return_if_fail (EGG_IS_TOOLBAR_EDITOR (editor));
 
   gtk_container_set_border_width (GTK_CONTAINER (editor), 12);
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -531,7 +572,7 @@ setup_editor (EggToolbarEditor *editor)
 static void
 egg_toolbar_editor_init (EggToolbarEditor *t)
 {
-  t->priv = g_new0 (EggToolbarEditorPrivate, 1);
+  t->priv = EGG_TOOLBAR_EDITOR_GET_PRIVATE (t);
 
   t->priv->merge = NULL;
   t->priv->default_actions_list = NULL;
@@ -540,7 +581,7 @@ egg_toolbar_editor_init (EggToolbarEditor *t)
   setup_editor (t);
 }
 
-static void
+void
 egg_toolbar_editor_add_action (EggToolbarEditor *editor,
 			       const char       *action_name)
 {
@@ -551,6 +592,8 @@ egg_toolbar_editor_add_action (EggToolbarEditor *editor,
 
 	editor->priv->default_actions_list = g_list_append
 		(editor->priv->default_actions_list, action);
+
+	sort_list (editor);
 }
 
 static void
@@ -631,6 +674,7 @@ egg_toolbar_editor_load_actions (EggToolbarEditor *editor,
 		(editor->priv->actions_list, action);
         }
     }
+  sort_list (editor);
 
   update_editor_sheet (editor);
 }
