@@ -15,13 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.h,v 1.2 2003/01/07 20:34:09 mpeseng Exp $
+ *  $Id: ephy-file-helpers.h,v 1.8 2004/01/25 00:38:46 marco Exp $
  */
 
 #ifndef EPHY_FILE_HELPERS_H
 #define EPHY_FILE_HELPERS_H
 
 #include <glib.h>
+#include <libxml/tree.h>
 
 G_BEGIN_DECLS
 
@@ -33,9 +34,9 @@ void        ephy_file_helpers_init       (void);
 
 void        ephy_file_helpers_shutdown   (void);
 
-void	    ephy_file_launch_application (const char *command_string,
-					  const char *parameter,
-					  gboolean use_terminal);
+char       *ephy_file_downloads_dir      (void);
+
+const char *ephy_file_tmp_dir	 	 (void);
 
 char       *ephy_file_tmp_filename	 (const char *base,
 					  const char *extension);
@@ -43,8 +44,13 @@ char       *ephy_file_tmp_filename	 (const char *base,
 void	    ephy_ensure_dir_exists       (const char *dir);
 
 GSList     *ephy_file_find               (const char *path,
-	                                  const char *fname,
-	                                  gint maxdepth);
+				          const char *fname,
+				          gint maxdepth);
+
+gboolean    ephy_file_switch_temp_file   (const char *filename,
+					  const char *filename_temp);
+
+void	    ephy_file_delete_on_exit	 (const char *path);
 
 G_END_DECLS
 
