@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-export.c,v 1.11 2003/12/16 17:29:43 marco Exp $
+ *  $Id: ephy-bookmarks-export.c,v 1.14 2004/01/28 18:55:53 chpe Exp $
  */
 
 #include "ephy-bookmarks-export.h"
@@ -94,7 +94,11 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 
 	/* FIXME: do we want to turn on compression here? */
 	writer = xmlNewTextWriterFilename (tmp_file, 0);
-	if (writer == NULL) return;
+	if (writer == NULL)
+	{
+		g_free (tmp_file);
+		return;
+	}
 
 	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
 	if (ret < 0) goto out;
@@ -150,6 +154,9 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 
 		kid = g_ptr_array_index (children, i);
 
+		ret = xmlTextWriterStartElementNS (writer, "rdf", "li", NULL);
+		if (ret < 0) break;
+
 		smart_url = ephy_node_has_child (smart_bmks, kid);
 		url = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_LOCATION);
@@ -170,16 +177,9 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 			}
 		}
 
-		if (link == NULL)
-		{
-			link = g_strdup (url);
-		}
-
-		ret = xmlTextWriterStartElementNS (writer, "rdf", "li", NULL);
-		if (ret < 0) break;
-
 		ret = xmlTextWriterWriteAttributeNS
-			(writer, "rdf", "about", NULL, link);
+			(writer, "rdf", "resource", NULL, link ? link : url);
+		g_free (link);
 		if (ret < 0) break;
 
 		ret = xmlTextWriterEndElement (writer); /* rdf:li */
@@ -278,6 +278,8 @@ out:
 		}
 	}
 
+	g_free (tmp_file);
+
 	STOP_PROFILER ("Exporting as RDF")
 
 	LOG ("Exporting as RDF %s.", ret >= 0 ? "succeeded" : "FAILED")
