@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.82 2004/01/22 19:23:06 clahey Exp $
+ *  $Id: ephy-shell.c,v 1.83 2004/02/01 22:52:10 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -710,8 +710,12 @@ ephy_shell_get_extensions_manager (EphyShell *es)
 
 	if (es->priv->extensions_manager == NULL)
 	{
-		/* Instantiate extensions manager; this will load the extensions */
+		/* Instantiate extensions manager */
 		es->priv->extensions_manager = ephy_extensions_manager_new ();
+
+		/* load the extensions */
+		ephy_extensions_manager_load_dir (es->priv->extensions_manager,
+						  EXTENSIONS_DIR);
 	}
 
 	return G_OBJECT (es->priv->extensions_manager);
