@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tabs-menu.c,v 1.21 2003/12/10 18:44:10 chpe Exp $
+ *  $Id: ephy-tabs-menu.c,v 1.23 2004/01/25 13:55:37 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -96,13 +96,13 @@ ephy_tabs_menu_get_type (void)
 }
 
 static void
-tab_added_cb (EphyNotebook *notebook, GtkWidget *child, EphyTabsMenu *menu)
+tab_added_cb (EphyNotebook *notebook, EphyEmbed *embed, EphyTabsMenu *menu)
 {
 	EphyTab *tab;
 	GtkAction *action;
-                                                                                                                             
-        g_return_if_fail (EPHY_IS_EMBED (child));
-        tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
+
+        g_return_if_fail (EPHY_IS_EMBED (embed));
+	tab = ephy_tab_for_embed (embed);
 
 	action = GTK_ACTION (ephy_tab_get_action (tab));
 	gtk_action_group_add_action (menu->priv->action_group, action);
@@ -111,13 +111,13 @@ tab_added_cb (EphyNotebook *notebook, GtkWidget *child, EphyTabsMenu *menu)
 }
 
 static void
-tab_removed_cb (EphyNotebook *notebook, GtkWidget *child, EphyTabsMenu *menu)
+tab_removed_cb (EphyNotebook *notebook, EphyEmbed *embed, EphyTabsMenu *menu)
 {
 	EphyTab *tab;
 	GtkAction *action;
                                                                                                                              
-	g_return_if_fail (EPHY_IS_EMBED (child));
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
+        g_return_if_fail (EPHY_IS_EMBED (embed));
+	tab = ephy_tab_for_embed (embed);
 
 	action = GTK_ACTION (ephy_tab_get_action (tab));
 	gtk_action_group_remove_action (menu->priv->action_group, action);
@@ -235,7 +235,7 @@ tab_set_action_accelerator (GtkActionGroup *action_group,
 			    guint tab_number)
 {
 	const char *action_name, *action_group_name;
-	char *accel_path = NULL;
+	char *accel_path;
 	char accel[7];
 	gint accel_number;
 	guint accel_key;
@@ -269,6 +269,8 @@ tab_set_action_accelerator (GtkActionGroup *action_group,
 	{
 		gtk_accel_map_change_entry (accel_path, 0, 0, TRUE);
 	}
+
+	g_free (accel_path);
 }
 
 void
